{*************
 *
 * Copyright (c) 1997 CEO Systems Inc.
 *
 * Project....: CEO Library
 * File.......: Base64.pas
 * Author.....: Anton A. Mints
 * Date.......: 20 September 1997
 * Description: Base64 (MIME) conversion functions 
 *              
 * $Archive: /CEO/VCL/Base64.pas $ 
 *              
 * $History: Base64.pas $ 
 * 
 * *****************  Version 1  *****************
 * User: Anton        Date: 9/20/97    Time: 5:12p
 * Created in $/CEO/VCL
 *              
 *************}

unit Base64;

interface

uses
  Windows, Classes, SysUtils;


procedure toBase64 (Src, Dst: TStream);


implementation

var
  EncodingTable: array[0..63] of Char;


{*************
* Name.......: LoadBase64EncodeTable
*}
procedure LoadBase64EncodeTable;
begin
   EncodingTable[0] := 'A';
   EncodingTable[1] := 'B';
   EncodingTable[2] := 'C';
   EncodingTable[3] := 'D';
   EncodingTable[4] := 'E';
   EncodingTable[5] := 'F';
   EncodingTable[6] := 'G';
   EncodingTable[7] := 'H';
   EncodingTable[8] := 'I';
   EncodingTable[9] := 'J';
   EncodingTable[10] := 'K';
   EncodingTable[11] := 'L';
   EncodingTable[12] := 'M';
   EncodingTable[13] := 'N';
   EncodingTable[14] := 'O';
   EncodingTable[15] := 'P';
   EncodingTable[16] := 'Q';
   EncodingTable[17] := 'R';
   EncodingTable[18] := 'S';
   EncodingTable[19] := 'T';
   EncodingTable[20] := 'U';
   EncodingTable[21] := 'V';
   EncodingTable[22] := 'W';
   EncodingTable[23] := 'X';
   EncodingTable[24] := 'Y';
   EncodingTable[25] := 'Z';
   EncodingTable[26] := 'a';
   EncodingTable[27] := 'b';
   EncodingTable[28] := 'c';
   EncodingTable[29] := 'd';
   EncodingTable[30] := 'e';
   EncodingTable[31] := 'f';
   EncodingTable[32] := 'g';
   EncodingTable[33] := 'h';
   EncodingTable[34] := 'i';
   EncodingTable[35] := 'j';
   EncodingTable[36] := 'k';
   EncodingTable[37] := 'l';
   EncodingTable[38] := 'm';
   EncodingTable[39] := 'n';
   EncodingTable[40] := 'o';
   EncodingTable[41] := 'p';
   EncodingTable[42] := 'q';
   EncodingTable[43] := 'r';
   EncodingTable[44] := 's';
   EncodingTable[45] := 't';
   EncodingTable[46] := 'u';
   EncodingTable[47] := 'v';
   EncodingTable[48] := 'w';
   EncodingTable[49] := 'x';
   EncodingTable[50] := 'y';
   EncodingTable[51] := 'z';
   EncodingTable[52] := '0';
   EncodingTable[53] := '1';
   EncodingTable[54] := '2';
   EncodingTable[55] := '3';
   EncodingTable[56] := '4';
   EncodingTable[57] := '5';
   EncodingTable[58] := '6';
   EncodingTable[59] := '7';
   EncodingTable[60] := '8';
   EncodingTable[61] := '9';
   EncodingTable[62] := '+';
   EncodingTable[63] := '/';
end;


{*************
* Name.......: uuENC
*}
function uuENC (OutByte: Byte): Char;
begin
   uuENC := Char((EncodingTable[((OutByte) and $3F)]));
end;


{*************
* Name.......: toBase64
* Description: Encode Srt stream into the Dst stream.
*}
procedure toBase64 (Src, Dst: TStream);

  procedure WriteLn (Line: String);
  var
    LineBuf: array [0..255] of Char;
  const
    CRLF = #13#10;
  begin
    StrCopy (LineBuf, PChar(Line + CRLF));
    Dst.Write (LineBuf, StrLen (LineBuf));
  end;

var
  CurLine: array[1..45] of Byte;
  CurByte: Byte;
  CurPos: Integer;
  EncodedLine: String;
  NumRead: Integer;
  EOF: Boolean;

begin
  EOF := False;

  // Load base64 code table
  LoadBase64EncodeTable;

  // Process all the source stream
  while not ( EOF ) do
  begin
    NumRead := Src.Read (CurLine, 45);
    EOF := ( NumRead <> 45 );

    EncodedLine := '';
    CurPos := 1;

    // Convert three bytes sequence to the four 6-bit sequence
    while ( CurPos <= (NumRead - 2) ) do
    begin
      CurByte := (CurLine[CurPos] SHR 2);
      EncodedLine := EncodedLine + uuENC(CurByte);

      CurByte := (( (CurLine[CurPos] and $03) SHL 4) or ( (CurLine[(CurPos + 1)] and $F0 ) SHR 4));
      EncodedLine := EncodedLine + uuENC(CurByte);

      CurByte := (( (CurLine[(CurPos + 1)] and $0F) SHL 2) or ( (CurLine[(CurPos + 2)] and $C0) SHR 6));
      EncodedLine := EncodedLine + uuENC(CurByte);

      CurByte := (CurLine[(CurPos + 2)] and $3F);
      EncodedLine := EncodedLine + uuENC(CurByte);

      CurPos := CurPos + 3;
    end;

    // Convert uncomplete byte sequence (less than three)
    if ( CurPos <= NumRead ) then
    begin
      CurByte := (CurLine[CurPos] SHR 2);
      EncodedLine := EncodedLine + uuENC(CurByte);

      // If there is only single byte
      if ( CurPos = NumRead ) then
        begin
          // convert the part of the single byte
          CurByte := ((CurLine[CurPos] and $03) SHL 4);
          EncodedLine := EncodedLine + uuENC(CurByte);
          EncodedLine := EncodedLine + '=';
        end
      // or two bytes
      else
        begin
          // convert the combination of the first and secont
          CurByte := (((CurLine[CurPos] and $03) SHL 4) or
                      ((CurLine[(CurPos + 1)] and $0F) SHR 4));
          EncodedLine := EncodedLine + uuENC(CurByte);

          // convert the rest of the second byte
          CurByte := ((CurLine[(CurPos + 1)] and $0F) SHL 2);
          EncodedLine := EncodedLine + uuENC(CurByte);
        end;

      EncodedLine := EncodedLine + '=';

    end;
    
    WriteLn (EncodedLine);
    
  end;

end;


end.

